@@ -1,21 +1,21 @@
-/* Copyright (C) 1995-1997, 2000, 2001, 2003, 2004, 2008-2011,
- *   2014 Free Software Foundation, Inc.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1997,2000-2001,2003-2004,2008-2011,2014,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -23,16 +23,21 @@
 # include <config.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/eq.h"
-
-#include "libguile/validate.h"
-#include "libguile/list.h"
-#include "libguile/eval.h"
-
 #include <stdarg.h>
 
+#include "boolean.h"
+#include "eq.h"
+#include "eval.h"
+#include "gsubr.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "procs.h"
+
+#include "list.h"
+
+
 
+
 /* creating lists */
 
 #define SCM_I_CONS(cell, x, y)                          \
@@ -219,7 +224,7 @@ SCM_DEFINE (scm_length, "length", 1, 0, 0,
 
 SCM_DEFINE (scm_append, "append", 0, 0, 1, 
             (SCM args),
-	    "Return a list consisting of the elements the lists passed as\n"
+	    "Return a list consisting of the elements of the lists passed as\n"
 	    "arguments.\n"
 	    "@lisp\n"
 	    "(append '(x) '(y))          @result{}  (x y)\n"
@@ -303,7 +308,7 @@ SCM_DEFINE (scm_append_x, "append!", 0, 0, 1,
 
 SCM_DEFINE (scm_last_pair, "last-pair", 1, 0, 0, 
            (SCM lst),
-	    "Return the last pair in @var{lst}, signalling an error if\n"
+	    "Return the last pair in @var{lst}, signaling an error if\n"
 	    "@var{lst} is circular.")
 #define FUNC_NAME s_scm_last_pair
 {
@@ -391,14 +396,14 @@ SCM_DEFINE (scm_reverse_x, "reverse!", 1, 1, 0,
   while (scm_is_pair (lst))
     {
       SCM old_tail = SCM_CDR (lst);
-      SCM_SETCDR (lst, tail);
+      scm_set_cdr_x (lst, tail);
       tail = lst;
       lst = old_tail;
     }
 
   if (SCM_LIKELY (SCM_NULL_OR_NIL_P (lst)))
     {
-      SCM_SETCDR (old_lst, new_tail);
+      scm_set_cdr_x (old_lst, new_tail);
       return tail;
     }
 
@@ -454,7 +459,7 @@ SCM_DEFINE (scm_list_set_x, "list-set!", 3, 0, 0,
   unsigned long int i = scm_to_ulong (k);
   while (scm_is_pair (lst)) {
     if (i == 0) {
-      SCM_SETCAR (lst, val);
+      scm_set_car_x (lst, val);
       return val;
     } else {
       --i;
@@ -500,7 +505,7 @@ SCM_DEFINE (scm_list_cdr_set_x, "list-cdr-set!", 3, 0, 0,
   size_t i = scm_to_size_t (k);
   while (scm_is_pair (lst)) {
     if (i == 0) {
-      SCM_SETCDR (lst, val);
+      scm_set_cdr_x (lst, val);
       return val;
     } else {
       --i;
@@ -1005,11 +1010,5 @@ SCM_DEFINE (scm_filter_x, "filter!", 2, 0, 0,
 void
 scm_init_list ()
 {
-#include "libguile/list.x"
+#include "list.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
