@@ -1,4 +1,4 @@
-/* Copyright 1995-1998,2000-2014,2018
+/* Copyright 1995-1998,2000-2014,2018-2019,2023-2024
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -32,6 +32,7 @@
 #include <stdlib.h>
 #include <string.h>   /* for memset used by FD_ZERO on Solaris 10 */
 #include <sys/time.h>
+#include <sys/select.h>                           /* from Gnulib */
 #include <unistd.h>
 
 #if HAVE_PTHREAD_NP_H
@@ -99,20 +100,16 @@ thread_mark (GC_word *addr, struct GC_ms_entry *mark_stack_ptr,
      but GC doesn't know to trace them (as they are pointerless), so we
      need to do that here.  See the comments at the top of libgc's
      gc_inline.h.  */
-  if (t->pointerless_freelists)
+  for (size_t n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
     {
-      size_t n;
-      for (n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
+      void *chain = t->pointerless_freelists[n];
+      if (chain)
         {
-          void *chain = t->pointerless_freelists[n];
-          if (chain)
-            {
-              /* The first link is already marked by the freelist vector,
-                 so we just have to mark the tail.  */
-              while ((chain = *(void **)chain))
-                mark_stack_ptr = GC_mark_and_push (chain, mark_stack_ptr,
-                                                   mark_stack_limit, NULL);
-            }
+          /* The first link is already marked by the thread itsel, so we
+             just have to mark the tail.  */
+          while ((chain = *(void **)chain))
+            mark_stack_ptr = GC_mark_and_push (chain, mark_stack_ptr,
+                                               mark_stack_limit, NULL);
         }
     }
 
@@ -358,6 +355,9 @@ scm_i_pthread_key_t scm_i_thread_key;
    itself in TLS (rather than a pointer to some malloc'd memory) is not
    possible since thread objects may live longer than the actual thread they
    represent.  */
+# ifdef __APPLE__ /* XXX: hack to address <https://bugs.gnu.org/60234> */
+SCM_INTERNAL
+# endif
 SCM_THREAD_LOCAL scm_thread *scm_i_current_thread = NULL;
 
 #endif /* SCM_HAVE_THREAD_STORAGE_CLASS */
@@ -443,12 +443,6 @@ guilify_self_2 (SCM dynamic_state)
   t->continuation_root = scm_cons (t->handle, SCM_EOL);
   t->continuation_base = t->base;
 
-  {
-    size_t size = SCM_INLINE_GC_FREELIST_COUNT * sizeof (void *);
-    t->freelists = scm_gc_malloc (size, "freelists");
-    t->pointerless_freelists = scm_gc_malloc (size, "atomic freelists");
-  }
-
   t->dynamic_state = scm_gc_typed_calloc (scm_t_dynamic_state);
   t->dynamic_state->thread_local_values = scm_c_make_hash_table (0);
   scm_set_current_dynamic_state (dynamic_state);
@@ -498,9 +492,8 @@ on_thread_exit (void *v)
   t->handle = SCM_PACK (0);
 
   /* If there's only one other thread, it could be the signal delivery
-     thread, so we need to notify it to shut down by closing its read pipe.
-     If it's not the signal delivery thread, then closing the read pipe isn't
-     going to hurt.  */
+     thread, in which case we should shut it down also by closing its
+     read pipe.  */
   if (thread_count <= 1)
     scm_i_close_signal_pipe ();
 
@@ -508,8 +501,8 @@ on_thread_exit (void *v)
 
   /* Although this thread has exited, the thread object might still be
      alive.  Release unused memory.  */
-  t->freelists = NULL;
-  t->pointerless_freelists = NULL;
+  for (size_t n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
+    t->freelists[n] = t->pointerless_freelists[n] = NULL;
   t->dynamic_state = NULL;
   t->dynstack.base = NULL;
   t->dynstack.top = NULL;
@@ -831,9 +824,9 @@ scm_spawn_thread (scm_t_catch_body body, void *body_data,
 {
   SCM body_closure, handler_closure;
 
-  body_closure = scm_i_make_catch_body_closure (body, body_data);
+  body_closure = scm_c_make_thunk (body, body_data);
   handler_closure = handler == NULL ? SCM_UNDEFINED :
-    scm_i_make_catch_handler_closure (handler, handler_data);
+    scm_i_make_catch_handler (handler, handler_data);
 
   return scm_call_with_new_thread (body_closure, handler_closure);
 }
@@ -908,7 +901,7 @@ enum scm_mutex_kind {
   /* An unowned mutex is like a standard mutex, except that it can be
      unlocked by any thread.  A corrolary of this behavior is that a
      thread's attempt to lock a mutex that it already owns will block
-     instead of signalling an error, as it could be that some other
+     instead of signaling an error, as it could be that some other
      thread unlocks the mutex, allowing the owner thread to proceed.
      This kind of mutex is a bit strange and is here for use by
      SRFI-18.  */
@@ -1032,14 +1025,7 @@ lock_mutex (enum scm_mutex_kind kind, struct scm_mutex *m,
 
         if (err == 0)
           {
-            if (scm_is_eq (m->owner, SCM_BOOL_F))
-              {
-                m->owner = current_thread->handle;
-                scm_i_pthread_mutex_unlock (&m->lock);
-                return SCM_BOOL_T;
-              }
-            else
-              continue;
+            goto maybe_acquire;
           }
         else if (err == ETIMEDOUT)
           {
@@ -1051,7 +1037,7 @@ lock_mutex (enum scm_mutex_kind kind, struct scm_mutex *m,
             scm_i_pthread_mutex_unlock (&m->lock);
             scm_async_tick ();
             scm_i_scm_pthread_mutex_lock (&m->lock);
-            continue;
+            goto maybe_acquire;
           }
         else
           {
@@ -1060,6 +1046,14 @@ lock_mutex (enum scm_mutex_kind kind, struct scm_mutex *m,
             errno = err;
             SCM_SYSERROR;
           }
+
+      maybe_acquire:
+        if (scm_is_eq (m->owner, SCM_BOOL_F))
+          {
+            m->owner = current_thread->handle;
+            scm_i_pthread_mutex_unlock (&m->lock);
+            return SCM_BOOL_T;
+          }
       }
 }
 #undef FUNC_NAME
@@ -1175,7 +1169,7 @@ SCM_DEFINE (scm_unlock_mutex, "unlock-mutex", 1, 0, 0, (SCM mutex),
             "Unlocks @var{mutex}.  The calling thread must already hold\n"
             "the lock on @var{mutex}, unless the mutex was created with\n"
             "the @code{allow-external-unlock} option; otherwise an error\n"
-            "will be signalled.")
+            "will be signaled.")
 #define FUNC_NAME s_scm_unlock_mutex
 {
   struct scm_mutex *m;
@@ -1397,15 +1391,16 @@ timed_wait (enum scm_mutex_kind kind, struct scm_mutex *m, struct scm_cond *c,
 
 SCM_DEFINE (scm_timed_wait_condition_variable, "wait-condition-variable", 2, 1, 0,
 	    (SCM cond, SCM mutex, SCM timeout),
-"Wait until condition variable @var{cv} has been signalled.  While waiting, "
+"Wait until condition variable @var{cv} has been signaled.  While waiting, "
 "mutex @var{mx} is atomically unlocked (as with @code{unlock-mutex}) and "
 "is locked again when this function returns.  When @var{t} is given, "
 "it specifies a point in time where the waiting should be aborted.  It "
 "can be either a integer as returned by @code{current-time} or a pair "
 "as returned by @code{gettimeofday}.  When the waiting is aborted the "
-"mutex is locked and @code{#f} is returned.  When the condition "
-"variable is in fact signalled, the mutex is also locked and @code{#t} "
-"is returned. ")
+"mutex is locked and @code{#f} is returned.  After the condition "
+"variable is signaled, the mutex is locked and @code{#t} is returned.  "
+"@code{#t} may also be returned spuriously, so any relevant conditions "
+"should be re-checked.")
 #define FUNC_NAME s_scm_timed_wait_condition_variable
 {
   scm_t_timespec waittime_val, *waittime = NULL;
@@ -1686,18 +1681,17 @@ SCM_DEFINE (scm_all_threads, "all-threads", 0, 0, 0,
 	    "Return a list of all threads.")
 #define FUNC_NAME s_scm_all_threads
 {
-  /* We can not allocate while holding the thread_admin_mutex because
-     of the way GC is done.
-  */
-  int n = thread_count;
   scm_thread *t;
-  SCM list = scm_c_make_list (n, SCM_UNSPECIFIED), *l;
 
   scm_i_pthread_mutex_lock (&thread_admin_mutex);
-  l = &list;
+
+  int n = thread_count;
+  SCM list = scm_c_make_list (n, SCM_UNSPECIFIED);
+  SCM *l = &list;
+
   for (t = all_threads; t && n > 0; t = t->next_thread)
     {
-      if (t != scm_i_signal_delivery_thread)
+      if (!t->exited && !scm_i_is_signal_delivery_thread (t))
 	{
 	  SCM_SETCAR (*l, t->handle);
 	  l = SCM_CDRLOC (*l);
@@ -1705,7 +1699,9 @@ SCM_DEFINE (scm_all_threads, "all-threads", 0, 0, 0,
       n--;
     }
   *l = SCM_EOL;
+
   scm_i_pthread_mutex_unlock (&thread_admin_mutex);
+
   return list;
 }
 #undef FUNC_NAME
